package com.fjhx.customer.service.xiaoman.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.json.JsonMapper;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.customer.contants.XiaomanContant;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.entity.customer.po.CustomerUser;
import com.fjhx.customer.entity.xiaoman.po.XiaomanCustomer;
import com.fjhx.customer.entity.xiaoman.po.XiaomanCustomerInfoJson;
import com.fjhx.customer.entity.xiaoman.vo.CustomerApiVo;
import com.fjhx.customer.entity.xiaoman.vo.CustomerInfoVo;
import com.fjhx.customer.handle.HandleXiaomanData;
import com.fjhx.customer.handle.R;
import com.fjhx.customer.mapper.xiaoman.XiaomanCustomerMapper;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.customer.service.customer.CustomerUserService;
import com.fjhx.customer.service.xiaoman.XiaomanApiService;
import com.fjhx.customer.service.xiaoman.XiaomanCustomerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.entity.dict.dto.DictTenantDataSelectDto;
import com.fjhx.tenant.entity.dict.vo.DictTenantDataVo;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.impl.SysUserServiceImpl;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.customer.entity.xiaoman.vo.XiaomanCustomerVo;
import com.fjhx.customer.entity.xiaoman.dto.XiaomanCustomerSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.customer.entity.xiaoman.dto.XiaomanCustomerDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ruoyi.common.utils.wrapper.IWrapper.getWrapper;

/**
 * <p>
 * 小满客户表 服务实现类
 * </p>
 *
 * @author lqh
 * @since 2024-04-06
 */
@Service
public class XiaomanCustomerServiceImpl extends ServiceImpl<XiaomanCustomerMapper, XiaomanCustomer> implements XiaomanCustomerService {

    @Resource
    private XiaomanCustomerInfoJsonServiceImpl xiaomanCustomerInfoJsonService;

    @Resource
    private CustomerService customerService;

    @Resource
    private CustomerUserService customerUserService;

    @Resource
    private SysUserServiceImpl sysUserService;
    @Resource
    private XiaomanApiService xiaomanApiService;

    @Override
    public List<XiaomanCustomerVo> getList(XiaomanCustomerSelectDto dto) {
        IWrapper<XiaomanCustomer> wrapper = getWrapper();
        wrapper.orderByDesc("xc", XiaomanCustomer::getCompanyId);
        List<XiaomanCustomerVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<XiaomanCustomerVo> getPage(XiaomanCustomerSelectDto dto) {
        IWrapper<XiaomanCustomer> wrapper = getWrapper();
        wrapper.orderByDesc("xc", XiaomanCustomer::getCompanyId);
        Page<XiaomanCustomerVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public XiaomanCustomerVo detail(Long id) {
        XiaomanCustomer XiaomanCustomer = this.getById(id);
        XiaomanCustomerVo result = BeanUtil.toBean(XiaomanCustomer, XiaomanCustomerVo.class);
        return result;
    }

    @Override
    public void add(XiaomanCustomerDto xiaomanCustomerDto) {
        this.save(xiaomanCustomerDto);
    }

    @Override
    public void edit(XiaomanCustomerDto xiaomanCustomerDto) {
        this.updateById(xiaomanCustomerDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public void handleSaveOrUpdate(List<CustomerApiVo> customerApiVoList, Set<Long> allCustomer) {
        List<XiaomanCustomer> xiaomanCustomerList = new ArrayList<>(customerApiVoList);
        for (XiaomanCustomer customer : xiaomanCustomerList){
            try {
                if (allCustomer.contains(customer.getCompanyId())){

                    //TODO 获取详情
                    String customerDetailJson = xiaomanApiService.getCustomerDetail(customer.getCompanyId());
                    XiaomanCustomerInfoJson xiaomanCustomerInfoJson = new XiaomanCustomerInfoJson();
                    xiaomanCustomerInfoJson.setCompanyId(customer.getCompanyId());
                    xiaomanCustomerInfoJson.setJson(customerDetailJson);
                    xiaomanCustomerInfoJsonService.updateById(xiaomanCustomerInfoJson);
                    //TODO 更新数据
                    CustomerInfoVo infoVo = HandleXiaomanData.handleDate(customerDetailJson, new TypeReference<R<CustomerInfoVo>>() {
                    });
                    Long s = customerConversion(infoVo);
                    customer.setCustomerId(s);
                    updateById(customer);
                }else {

                    //TODO 获取详情
                    String customerDetailJson = xiaomanApiService.getCustomerDetail(customer.getCompanyId());
                    XiaomanCustomerInfoJson xiaomanCustomerInfoJson = new XiaomanCustomerInfoJson();
                    xiaomanCustomerInfoJson.setCompanyId(customer.getCompanyId());
                    xiaomanCustomerInfoJson.setJson(customerDetailJson);
                    xiaomanCustomerInfoJsonService.save(xiaomanCustomerInfoJson);

                    //TODO 更新数据
                    CustomerInfoVo infoVo = HandleXiaomanData.handleDate(customerDetailJson, new TypeReference<R<CustomerInfoVo>>() {
                    });
                    //小满客户 =》 客户
                    Long s = customerConversion(infoVo);
                    customer.setCustomerId(s);
                    save(customer);
                    System.out.println("2");
                }
            }catch (Exception e){
               e.printStackTrace();
            }
        }


    }

    /**
     * 小满客户 =》 客户
     * @param vo
     */
    public Long customerConversion(CustomerInfoVo vo) {
        SecurityUtils.setTenantId(XiaomanContant.TENANT_ID);
        List<Customer> list = customerService.lambdaQuery()
                .eq(StringUtils.isNotEmpty(vo.getName()), Customer::getName, vo.getName())
                .or()
                .eq(StringUtils.isNotEmpty(vo.getShortName()), Customer::getName, vo.getShortName())
                .or()
                .eq(Objects.nonNull(vo.getCompanyId()), Customer::getCompanyId, vo.getCompanyId())
                .list();

        if (!list.isEmpty()){
            for (Customer customer :list){
                //小满公司ID
                customer.setCompanyId(vo.getCompanyId());
                //公司编号
                customer.setCustomerCode(vo.getSerialId());
                //业务员ID
                if (!vo.getOwner().isEmpty()){
                    DynamicDataSourceContextHolder.push(SourceConstant.BASE);
                    SysUser sysUser = sysUserService.lambdaQuery().eq(SysUser::getNickNameEn, vo.getOwner().get(0).getName()).one();
                    if (Objects.nonNull(sysUser)){
                        customer.setUserId(sysUser.getUserId());
                    }
                    DynamicDataSourceContextHolder.poll();
                }
                //TODO 标签tag  customer_tag
                vo.getTag().stream().map(CustomerInfoVo.Tag::getTagName).collect(Collectors.joining(","));

                //客户名称
                customer.setName(vo.getName());
                //客户简称
                customer.setShortName(vo.getShortName());

                //TODO 客户来源source  customer_source
                String.join(",",vo.getOriginList());

                //客户规模
                customer.setScaleId(vo.getScaleId());
                //TODO 客户来源status  customer_status
                vo.getTrailStatus();

                //公司网址
                customer.setHomepage(vo.getHomepage());
                //小满传真
                customer.setFax(vo.getFax());
                //小满电话区号
                customer.setTelAreaCode(vo.getTelAreaCode());
                //小满电话
                customer.setTel(vo.getTel());
                //小满备注
                customer.setRemark(vo.getRemark());


                //业务员ID
                if (!vo.getCustomers().isEmpty()){
                    Map<String, CustomerUser> collect = customerUserService.lambdaQuery()
                            .eq(CustomerUser::getCustomerId, customer.getId())
                            .list().stream().collect(Collectors.toMap(CustomerUser::getEmail, Function.identity()));

                    Set<String > str = new HashSet<>();
                    for (CustomerInfoVo.Customer cus  :vo.getCustomers()){
                        if ( !str.contains(customer.getId()+"_"+ cus.getEmail())){
                            //如果email匹配成功新增 ，否则修改
                            if (collect.containsKey(cus.getEmail()) ){
                                List<Map> maps = new ArrayList<>();
                                CustomerUser customerUser = collect.get(cus.getEmail());
                                if (StringUtils.isNotEmpty(customerUser.getContactJson())) {
                                    maps = JSON.parseArray(customerUser.getContactJson(), Map.class);
                                }
                                for (CustomerInfoVo.Contact contact: cus.getContact()){
                                    if (StringUtils.isNotEmpty(customerUser.getContactJson())){
                                        for (Map map:maps){
                                            if (!contact.getValue().equals(map.get("contactNo")) && !contact.getType().equals(map.get("type"))){
                                                Map map1 = new HashMap();
                                                map1.put("type",contact.getType());
                                                map1.put("contactNo",contact.getValue());
                                                maps.add(map1);
                                            }else {
                                                System.out.println();
                                            }
                                        }

                                    }else {
                                        Map map1 = new HashMap();
                                        map1.put("type",contact.getType());
                                        map1.put("contactNo",contact.getValue());
                                        maps.add(map1);
                                    }

                                }
                                customerUser.setEmail(cus.getEmail());
                                customerUser.setContactJson(JSON.toJSONString(maps));
                                str.add(customer.getId()+"_"+ cus.getEmail());
                                customerUserService.updateById(customerUser);
                            }else {

                                CustomerUser customerUser = new CustomerUser();
                                customerUser.setCustomerId(customer.getId());
                                customerUser.setName(cus.getName());
                                customerUser.setEmail(cus.getEmail());
                                List<Map> maps = new ArrayList<>();
                                if (! cus.getContact().isEmpty()){
                                    for (CustomerInfoVo.Contact contact : cus.getContact()) {
                                        Map map1 = new HashMap();
                                        map1.put("type", contact.getType());
                                        map1.put("contactNo", contact.getValue());
                                        maps.add(map1);
                                    }
                                }else{
                                    Map map1 = new HashMap();
                                    map1.put("type", "");
                                    map1.put("contactNo", "");
                                    maps.add(map1);
                                }
//                            customerUser.setPhone(cus.getTel());
                                customerUser.setContactJson(JSON.toJSONString(maps));

                                str.add(customer.getId()+"_"+ cus.getEmail());
                                customerUserService.save(customerUser);

                            }
                        }
                    }

                }

                customerService.updateById(customer);
            }

        }

        SecurityUtils.clearTenantId();
        if (!list.isEmpty()) {
            Long collect = list.stream().map(Customer::getId).collect(Collectors.toList()).get(0);
            return collect;
        }else {
            return null;
        }

    }


}
